@@ -11,6 +11,8 @@
 
 #include "spoa/spoa.hpp"
 
+#include "spoa_test_config.h"
+
 std::atomic<std::uint32_t> biosoup::Sequence::num_objects{0};
 
 namespace spoa {
@@ -18,7 +20,7 @@ namespace test {
 
 class SpoaTest: public ::testing::Test {
  public:
-  void Setup(
+  void Initialize(
       AlignmentType type,
       std::int8_t m,
       std::int8_t n,
@@ -27,7 +29,7 @@ class SpoaTest: public ::testing::Test {
       std::int8_t q,
       std::int8_t c,
       bool quality) {
-    auto p = bioparser::Parser<biosoup::Sequence>::Create<bioparser::FastqParser>(TEST_DATA);  // NOLINT
+    auto p = bioparser::Parser<biosoup::Sequence>::Create<bioparser::FastqParser>(SPOA_TEST_DATA);  // NOLINT
     s = p->Parse(-1);
     EXPECT_EQ(55, s.size());
 
@@ -109,17 +111,10 @@ TEST(SpoaAlignmentTest, LargeInput) {
         std::string(exception.what()).substr(11),
         "AlignmentEngine::Prealloc] error: too large sequence!");
   }
-  try {
-    ae->Prealloc((1ULL << 31) - 1, -1);
-  } catch (std::invalid_argument& exception) {
-    EXPECT_EQ(
-        std::string(exception.what()).substr(11),
-        "AlignmentEngine::Prealloc] error: insufficient memory!");
-  }
 }
 
 TEST_F(SpoaTest, Clear) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, false);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, false);
   Align();
   auto c = gr.GenerateConsensus();
 
@@ -130,7 +125,7 @@ TEST_F(SpoaTest, Clear) {
 
 #ifdef SPOA_USE_CEREAL
 TEST_F(SpoaTest, Archive) {
-  Setup(AlignmentType::kNW, 2, -5, -2, -2, -2, -2, true);
+  Initialize(AlignmentType::kNW, 2, -5, -2, -2, -2, -2, true);
 
   {
     std::ofstream os("spoa.test.cereal");
@@ -152,7 +147,7 @@ TEST_F(SpoaTest, Archive) {
 #endif
 
 TEST_F(SpoaTest, Local) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, false);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, false);
   Align();
 
   std::string c =
@@ -168,7 +163,7 @@ TEST_F(SpoaTest, Local) {
 }
 
 TEST_F(SpoaTest, LocalAffine) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -6, -8, -6, false);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -6, -8, -6, false);
   Align();
 
   std::string c =
@@ -184,7 +179,7 @@ TEST_F(SpoaTest, LocalAffine) {
 }
 
 TEST_F(SpoaTest, LocalConvex) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -6, -10, -2, false);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -6, -10, -2, false);
   Align();
 
   std::string c =
@@ -200,7 +195,7 @@ TEST_F(SpoaTest, LocalConvex) {
 }
 
 TEST_F(SpoaTest, LocalWithQualities) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, true);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -8, -8, -8, true);
   Align();
 
   std::string c =
@@ -216,7 +211,7 @@ TEST_F(SpoaTest, LocalWithQualities) {
 }
 
 TEST_F(SpoaTest, LocalAffineWithQualities) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -6, -8, -6, true);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -6, -8, -6, true);
   Align();
 
   std::string c =
@@ -232,7 +227,7 @@ TEST_F(SpoaTest, LocalAffineWithQualities) {
 }
 
 TEST_F(SpoaTest, LocalConvexWithQualities) {
-  Setup(AlignmentType::kSW, 5, -4, -8, -6, -10, -2, true);
+  Initialize(AlignmentType::kSW, 5, -4, -8, -6, -10, -2, true);
   Align();
 
   std::string c =
@@ -248,7 +243,7 @@ TEST_F(SpoaTest, LocalConvexWithQualities) {
 }
 
 TEST_F(SpoaTest, Global) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -8, -8, -8, false);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -8, -8, -8, false);
   Align();
 
   std::string c =
@@ -264,7 +259,7 @@ TEST_F(SpoaTest, Global) {
 }
 
 TEST_F(SpoaTest, GlobalAffine) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -6, -8, -6, false);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -6, -8, -6, false);
   Align();
 
   std::string c =
@@ -280,7 +275,7 @@ TEST_F(SpoaTest, GlobalAffine) {
 }
 
 TEST_F(SpoaTest, GlobalConvex) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -6, -10, -2, false);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -6, -10, -2, false);
   Align();
 
   std::string c =
@@ -296,7 +291,7 @@ TEST_F(SpoaTest, GlobalConvex) {
 }
 
 TEST_F(SpoaTest, GlobalWithQualities) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -8, -8, -8, true);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -8, -8, -8, true);
   Align();
 
   std::string c =
@@ -312,7 +307,7 @@ TEST_F(SpoaTest, GlobalWithQualities) {
 }
 
 TEST_F(SpoaTest, GlobalAffineWithQualities) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -6, -8, -6, true);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -6, -8, -6, true);
   Align();
 
   std::string c =
@@ -328,7 +323,7 @@ TEST_F(SpoaTest, GlobalAffineWithQualities) {
 }
 
 TEST_F(SpoaTest, GlobalConvexWithQualities) {
-  Setup(AlignmentType::kNW, 5, -4, -8, -6, -10, -2, true);
+  Initialize(AlignmentType::kNW, 5, -4, -8, -6, -10, -2, true);
   Align();
 
   std::string c =
@@ -344,7 +339,7 @@ TEST_F(SpoaTest, GlobalConvexWithQualities) {
 }
 
 TEST_F(SpoaTest, SemiGlobal) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -8, -8, -8, false);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -8, -8, -8, false);
   Align();
 
   std::string c =
@@ -360,7 +355,7 @@ TEST_F(SpoaTest, SemiGlobal) {
 }
 
 TEST_F(SpoaTest, SemiGlobalAffine) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -6, -8, -6, false);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -6, -8, -6, false);
   Align();
 
   std::string c =
@@ -376,7 +371,7 @@ TEST_F(SpoaTest, SemiGlobalAffine) {
 }
 
 TEST_F(SpoaTest, SemiGlobalConvex) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -6, -10, -2, false);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -6, -10, -2, false);
   Align();
 
   std::string c =
@@ -392,7 +387,7 @@ TEST_F(SpoaTest, SemiGlobalConvex) {
 }
 
 TEST_F(SpoaTest, SemiGlobalWithQualities) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -8, -8, -8, true);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -8, -8, -8, true);
   Align();
 
   std::string c =
@@ -408,7 +403,7 @@ TEST_F(SpoaTest, SemiGlobalWithQualities) {
 }
 
 TEST_F(SpoaTest, SemiGlobalAffineWithQualities) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -6, -8, -6, true);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -6, -8, -6, true);
   Align();
 
   std::string c =
@@ -424,7 +419,7 @@ TEST_F(SpoaTest, SemiGlobalAffineWithQualities) {
 }
 
 TEST_F(SpoaTest, SemiGlobalConvexWithQualities) {
-  Setup(AlignmentType::kOV, 5, -4, -8, -6, -10, -2, true);
+  Initialize(AlignmentType::kOV, 5, -4, -8, -6, -10, -2, true);
   Align();
 
   std::string c =
